@@ -1,8 +1,14 @@
+import base64
 import importlib
 import json
+import os
+import shlex
 
 import kubernetes
+from ocp_resources.image_content_source_policy import ImageContentSourcePolicy
 from ocp_resources.node import Node
+from ocp_resources.resource import ResourceEditor
+from ocp_resources.secret import Secret
 from ocp_wrapper_data_collector.data_collector import (
     get_data_collector_base_dir,
     get_data_collector_dict,
@@ -16,6 +22,7 @@ from ocp_utilities.exceptions import (
     NodeUnschedulableError,
     PodsFailedOrPendingError,
 )
+from ocp_utilities.utils import run_command
 
 
 LOGGER = get_logger(name=__name__)
@@ -288,3 +295,124 @@ def cluster_resource(base_class):
     """
     creator = DynamicClassCreator()
     return creator(base_class=base_class)
+
+
+def create_icsp_command(
+    image, source_url, folder_name, pull_secret=None, filter_options=""
+):
+    """
+        Create ImageContentSourcePolicy command.
+
+    Args:
+        image (str): name of image to be mirrored.
+        source_url (str): source url of image registry to which contents mirror.
+        folder_name (str): local path to store manifests.
+        pull_secret (str): Path to your registry credentials, default set to None(until passed)
+        filter_options (str): when filter passed it will choose image from multiple variants.
+
+    Returns:
+        str: base command to create icsp in the cluster.
+    """
+    base_command = (
+        f"oc adm catalog mirror {image} {source_url} --manifests-only "
+        f"--to-manifests {folder_name} {filter_options}"
+    )
+    if pull_secret:
+        base_command = f"{base_command} --registry-config={pull_secret}"
+    return base_command
+
+
+def generate_icsp_file(
+    folder_name, image, source_url, pull_secret=None, filter_options=""
+):
+    base_command = create_icsp_command(
+        image=image,
+        source_url=source_url,
+        folder_name=folder_name,
+        pull_secret=pull_secret,
+        filter_options=filter_options,
+    )
+    assert run_command(
+        command=shlex.split(base_command),
+        verify_stderr=False,
+    )[0]
+
+    icsp_file_path = os.path.join(folder_name, "ImageContentSourcePolicy.yaml")
+    assert os.path.isfile(
+        icsp_file_path
+    ), f"ICSP file does not exist in path {icsp_file_path}"
+
+    return icsp_file_path
+
+
+def create_icsp_from_file(icsp_file_path):
+    icsp = ImageContentSourcePolicy(yaml_file=icsp_file_path)
+    icsp.deploy()
+    return icsp
+
+
+def create_icsp(icsp_name, repository_digest_mirrors):
+    icsp = ImageContentSourcePolicy(
+        name=icsp_name, repository_digest_mirrors=repository_digest_mirrors
+    )
+    icsp.deploy()
+    return icsp
+
+
+def dict_base64_encode(_dict):
+    """
+    Encoding dict in base64
+
+    Args:
+        _dict (dict): data dict to be encoded
+
+    Returns:
+        str: given _dict encoded in base64
+    """
+    return base64.b64encode(json.dumps(_dict).encode("ascii")).decode("utf-8")
+
+
+def create_update_secret(secret_data_dict, name, namespace):
+    """
+    Update existing secret or create a new secret; secret type - dockerconfigjson
+
+    Args:
+        secret_data_dict (dict): Secret data to be added/created,
+            example: {"auths":{
+                    <registry_name>:
+                        {"auth": <auth_token>,
+                        "email": <auth_email>},
+                    ...,
+                    <registry_name>:
+                        {"auth": <auth_token>,
+                        "email": <auth_email>}
+                    }
+                }
+        name (str): Secret name
+        namespace (str): Secret namespace
+
+    Returns:
+        Secret: secret object with secret_data_dict content included
+    """
+    secret = Secret(name=name, namespace=namespace)
+    secret_key = ".dockerconfigjson"
+    auths_key = "auths"
+
+    if secret.exists:
+        old_secret_data_dict = json.loads(
+            base64.b64decode(secret.instance.data[secret_key])
+        )[auths_key]
+        old_secret_data_dict.update(secret_data_dict[auths_key])
+        secret_data_encoded = dict_base64_encode(
+            _dict={auths_key: old_secret_data_dict}
+        )
+
+        ResourceEditor(
+            patches={secret: {"data": {secret_key: secret_data_encoded}}}
+        ).update()
+
+        return secret
+
+    secret.data_dict = {secret_key: dict_base64_encode(_dict=secret_data_dict)}
+
+    return secret.deploy()
